<?php

namespace Drupal\entity_access_by_role_field\Plugin\Field\FieldWidget;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\entity_access_by_role_field\Helper\Constants;
use Drupal\Core\Field\Plugin\Field\FieldWidget\OptionsButtonsWidget;

/**
 * Plugin implementation of the 'entity_access_by_role_field' widget.
 *
 * @FieldWidget(
 *   id = "default_entity_access_by_role_field_widget",
 *   module = "entity_access_by_role_field",
 *   label = @Translation("Allow / Deny Roles Widget"),
 *   field_types = {
 *     "entity_access_by_role_field"
 *   },
 *   multiple_values = TRUE
 * )
 */
class DefaultEntityAccessRoleWidget extends OptionsButtonsWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'role_id' => ['administrator'],
      'access' => Constants::ACCESS_ALLOWED,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    // Set correct default value if NULL returned from core:
    $element['#default_value'] = $element['#default_value'] ?? $this->defaultSettings()['role_id'];

    $newElement = [];
    $newElement['details'] = [
      '#type' => 'details',
      '#title' => $element['#title'],
      '#open' => TRUE,
      '#tree' => TRUE,
      '#description' => $element['#description'],
    ];

    $operations = _entity_access_by_role_field_filter_operations($this->fieldDefinition->getSetting('operations'));
    $operationsString = implode(' ' . $this->t('or') . ' ', $operations);
    $newElement['details']['access'] = [
      '#title' => $this->t('Allow / restrict role access to @operations this entity:', ['@operations' => $operationsString]),
      '#type' => 'radios',
      '#options' => [
        Constants::ACCESS_ALLOWED => $this->t('Allow the selected below'),
        Constants::ACCESS_FORBIDDEN => $this->t('Restrict the selected below'),
      ],
      '#default_value' => $items[$delta]->access ?? $this->defaultSettings()['access'],
      '#required' => TRUE,
      '#description' => $this->t('Allow or restrict access by role for the selected operations: <strong>@operations</strong>', ['@operations' => $operationsString]),
    ];

    // We set a different title for the field and use the field name for
    // the details wrapper.
    $element['#title'] = $this->t('When the user has the following roles:');
    $element['#description'] = $this->t("Select the roles that should be allowed / denied access. They are combined by OR. If no roles are selected the access will fallback to '@fallbackValue'.", ['@fallbackValue' => $this->fieldDefinition->getSetting('empty_roles_access_fallback')]);
    // Re-add the original element below the details wrapper:
    $newElement['details']['role_id'] = $element;

    return $newElement;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {

    /*
     * The submit handler expects a structure like in database:
     * [
     *    0 => [
     *      'role_id' => 'authenticated',
     *      'access' => 'allowed',
     *    ],
     *    1 => [
     *      'role_id' => 'administrator',
     *      'access' => 'allowed',
     *    ],
     *    ...
     * ]
     */

    $massagedValues = [];
    if (!empty($values)) {
      // Set roles by delta:
      $massagedValues = $values['details']['role_id'];
      // Append access value into each role array:
      foreach ($massagedValues as $key => $value) {
        $massagedValues[$key]['access'] = $values['details']['access'];
      }
    }
    return $massagedValues;
  }

}
