<?php

namespace Drupal\Tests\entity_access_by_role_field\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\field\Entity\FieldConfig;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessibleInterface;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\entity_access_by_role_field\Helper\Constants;

/**
 * This class provides methods for access testing entity_access_by_role_field.
 *
 * @group entity_access_by_role_field
 */
abstract class EntityAccessTestBase extends BrowserTestBase {

  /**
   * The field name to test.
   *
   * @var string
   */
  protected $fieldName = 'field_access';

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * An admin user with all permissions.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $adminUser;

  /**
   * A user with the permission to skip field check.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $globalPermissionUser;

  /**
   * An authenticated user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $authenticatedUser;

  /**
   * A simple authenticated user with a test_role.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $testRoleUser;

  /**
   * Creates an Access field on an entity bundle type.
   *
   * @param \Drupal\Core\Config\Entity\ConfigEntityInterface $entityTypeBundle
   *   The entityTypeBundle to add the field on.
   * @param string $fieldName
   *   The field name of the field.
   * @param array $operationDefaults
   *   The allowed operations to access the instance of the entityTypeBundle.
   * @param string $emptyRoleFallbackDefaults
   *   The emptyRoleFallback value if no roles are selected.
   * @param null|int $cardinality
   *   The parameter which defines if there are multiple roles selected.
   */
  protected function createAccessFieldTypeOnEntityBundle(ConfigEntityInterface $entityTypeBundle, string $fieldName, array $operationDefaults = [Constants::OPERATION_VIEW], string $emptyRoleFallbackDefaults = Constants::FALLBACK_NEUTRAL, $cardinality = NULL) {
    $entityType = $entityTypeBundle->getEntityType()->id();
    $bundle = $entityTypeBundle->id();

    // FIXME: $entityType results are WRONG!
    // @todo TODO: Find the right way to get "node" instead of node_type
    // and "taxonomy_term" instead of "taxonomy_vocabulary".
    // SUPER Dirty workaround to remove _type - as we don't know how to
    // get it without the suffix... but we didn't have the time:
    $entityType = str_replace('_type', '', $entityType);
    if ($entityType == 'taxonomy_vocabulary') {
      $entityType = 'taxonomy_term';
    }

    // Create field:
    FieldStorageConfig::create([
      'field_name' => $fieldName,
      'entity_type' => $entityType,
      'type' => Constants::FIELD_TYPE,
      'cardinality' => $cardinality ?? FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED,
    ])->save();

    // Create field instance:
    FieldConfig::create([
      'field_name' => $fieldName,
      'entity_type' => $entityType,
      'bundle' => $bundle,
      'settings' => [
        'operations' => $operationDefaults,
        'empty_roles_access_fallback' => $emptyRoleFallbackDefaults,
      ],
    ])->save();

    return $entityTypeBundle;
  }

  /**
   * Adds an entity access by role field to a bundle instance.
   *
   * @param \Drupal\Core\Entity\FieldableEntityInterface $entity
   *   The entity bundle instance to add the field on.
   * @param string $fieldName
   *   The field name.
   * @param array $roleIds
   *   The roleIds used in access.
   * @param string $access
   *   Role ids have access allowed or forbidden on the entity bundle instance.
   */
  protected function setEntityAccessByRoleFieldOnEntity(FieldableEntityInterface $entity, string $fieldName, array $roleIds, string $access) {
    $fieldValues = [];
    foreach ($roleIds as $roleid) {
      $fieldValues[] = [
        'role_id' => $roleid,
        'access' => $access,
      ];
    }
    $entity->set($fieldName, $fieldValues);
    return $entity;
  }

  /**
   * Asserts that access is allowed to access the entity for the operation.
   *
   * @param \Drupal\Core\Access\AccessibleInterface $entity
   *   The Entity to test the access on.
   * @param string|null $operation
   *   The allowed operation on the entity.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user to check the access on, NULL is current user.
   */
  protected static function assertAccessAllowed(AccessibleInterface $entity, ?string $operation = 'view', AccountInterface $account = NULL) {
    static::assertTrue($entity->access($operation, $account));
  }

  /**
   * Asserts that access is forbidden to access the entity for the operation.
   *
   * @param \Drupal\Core\Access\AccessibleInterface $entity
   *   The Entity to test the access on.
   * @param string|null $operation
   *   The allowed operation on the entity.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user to check the access on, NULL is current user.
   */
  protected static function assertAccessForbidden(AccessibleInterface $entity, ?string $operation = 'view', AccountInterface $account = NULL) {
    static::assertFalse($entity->access($operation, $account));
  }

}
