<?php

namespace Drupal\Tests\entity_access_by_role_field\Functional;

use Drupal\media\Entity\Media;
use Drupal\Tests\media\Traits\MediaTypeCreationTrait;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\entity_access_by_role_field\Helper\Constants;

/**
 * This class provides access tests for entity_access_by_role_field media.
 *
 * @group entity_access_by_role_field
 */
class MediaEntityAccessTests extends EntityAccessTestBase {
  use MediaTypeCreationTrait;

  /**
   * {@inheritDoc}
   */
  public function setUp(): void {
    parent::setUp();

    // Enable URLs for media:
    \Drupal::configFactory()
      ->getEditable('media.settings')
      ->set('standalone_url', TRUE)
      ->save(TRUE);

    $this->container->get('router.builder')->rebuild();

    // Create the adminUser:
    $this->createAdminRole('administrator', 'administrator');
    $this->adminUser = $this->createUser([
      'administer media',
      'administer media types',
      'view media',
    ]);
    $this->adminUser->addRole('administrator');
    $this->adminUser->save();

    // Create the global permissionUser:
    $this->globalPermissionUser = $this->createUser([
      'administer media',
      'administer media types',
      'view media',
      'entity_by_access_role_field.ignore_permission',
    ]);

    // Create the authenticatedUser:
    $this->authenticatedUser = $this->createUser([
      'administer media',
      'administer media types',
      'view media',
    ]);

    // Create the testRoleUser:
    $this->testRoleUser = $this->createUser([
      'administer media',
      'administer media types',
    ]);
    $this->createRole(['view media'], 'test_role', 'test_role');
    $this->testRoleUser->addRole('test_role');
    $this->testRoleUser->save();
  }

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'entity_access_by_role_field',
    'field',
    'media',
    // Needed for 'test' media types:
    'media_test_source',
  ];

  /**
   * Tests access on a allowed global media entity by an registered user.
   */
  public function testAccessOnAllowedGlobalMedia() {
    $this->drupalLogin($this->authenticatedUser);
    $media_type = $this->createMediaType('test');
    $this->createAccessFieldTypeOnEntityBundle($media_type, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    // Create media.
    $media = Media::create([
      'bundle' => $media_type->id(),
      'name' => 'Unnamed',
    ]);
    $media->save();
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($media, $this->fieldName, ['authenticated'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessAllowed($media);
  }

  /**
   * Tests access on a forbidden global media entity by an registered user.
   */
  public function testAccessOnForbiddenGlobalMedia() {
    $this->drupalLogin($this->authenticatedUser);
    $media_type = $this->createMediaType('test');
    $this->createAccessFieldTypeOnEntityBundle($media_type, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    // Create media.
    $media = Media::create([
      'bundle' => $media_type->id(),
      'name' => 'Unnamed',
    ]);
    $media->save();
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($media, $this->fieldName, ['authenticated'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($media);
  }

  /**
   * Tests access on a allowed user media entity by an registered user.
   */
  public function testAccessOnAllowedUserMedia() {
    $this->drupalLogin($this->authenticatedUser);
    $media_type = $this->createMediaType('test');
    $this->createAccessFieldTypeOnEntityBundle($media_type, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    // Create media.
    $media = Media::create([
      'bundle' => $media_type->id(),
      'name' => 'Test media created by the authenticated user',
      'uid' => $this->authenticatedUser->id(),
    ]);
    $media->save();
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($media, $this->fieldName, ['authenticated'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessAllowed($media);
  }

  /**
   * Tests access on a forbidden user media entity by an registered user.
   */
  public function testAccessOnForbiddenUserMedia() {
    $this->drupalLogin($this->authenticatedUser);
    $media_type = $this->createMediaType('test');
    $this->createAccessFieldTypeOnEntityBundle($media_type, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    // Create media.
    $media = Media::create([
      'bundle' => $media_type->id(),
      'name' => 'Test media created by the authenticated user',
      'uid' => $this->authenticatedUser->id(),
    ]);
    $media->save();
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($media, $this->fieldName, ['authenticated'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($media);
  }

}
