<?php

namespace Drupal\Tests\entity_access_by_role_field\Functional;

use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\entity_access_by_role_field\Helper\Constants;

/**
 * This class provides access tests for entity_access_by_role_field node.
 *
 * @group entity_access_by_role_field
 */
class NodeEntityAccessTests extends EntityAccessTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'entity_access_by_role_field',
    'node',
    'field',
  ];

  /**
   * {@inheritDoc}
   */
  public function setUp(): void {
    parent::setUp();

    // Create the adminUser:
    $adminRoleRid = $this->createAdminRole('administrator', 'administrator');
    $this->adminUser = $this->createUser([
      'administer content types',
      'administer nodes',
    ]);
    $this->adminUser->addRole($adminRoleRid);
    $this->adminUser->save();

    // Create the global permissionUser:
    $this->globalPermissionUser = $this->createUser([
      'administer content types',
      'administer nodes',
      'entity_by_access_role_field.ignore_permission',
    ]);

    // Create the authenticatedUser:
    $this->authenticatedUser = $this->createUser([
      'administer content types',
      'administer nodes',
    ]);

    // Create the testRoleUser:
    $this->testRoleUser = $this->createUser([
      'administer content types',
    ]);
    $testRoleRid = $this->createRole(['administer nodes'], 'test_role', 'test_role');
    $this->testRoleUser->addRole($testRoleRid);
    $this->testRoleUser->save();
  }

  /**
   * Test if global ignore_permission ignores field settings.
   */
  public function testGlobalPermissionOnForbiddenNode() {
    $this->drupalLogin($this->globalPermissionUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Check expected access BEFORE we add our field:
    $this->drupalGet('/node/' . $entity->id());
    $this->assertSession()->statusCodeEquals(200);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity);
  }

  /**
   * Test if accessing node as admin, ignores field settings.
   */
  public function testAdminAccessOnForbiddenNode() {
    $this->drupalLogin($this->adminUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['administrator'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity);
  }

  /**
   * Check access on a node which allows authenticated users.
   */
  public function testAccessOnAllowedNodeAsAuth() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity);
    // Now test with get request:
    $session = $this->assertSession();
    $this->drupalGet('/node/' . $entity->id());
    $session->statusCodeEquals(200);
  }

  /**
   * Check access on a node which forbids authenticated users.
   */
  public function testAccessOnForbiddenNodeAsAuth() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Check expected access BEFORE we add our field:
    $this->drupalGet('/node/' . $entity->id());
    $this->assertSession()->statusCodeEquals(200);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity);
  }

  /**
   * Check access on a node which allows anonymous users, as a anonymous user.
   */
  public function testAccessOnAllowedNodeAsGuest() {
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['anonymous'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity);
    // Now test with get request:
    $session = $this->assertSession();
    $this->drupalGet('/node/' . $entity->id());
    $session->statusCodeEquals(200);
  }

  /**
   * Check access on a node which forbids anonymous users, as a anonymous user.
   *
   * Special test, testing 2 articles.
   */
  public function testAccessOnForbiddenNodeAsGuest() {
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    // We create two entities here, as the access results otherwise
    // seem to be cached wrongly!
    $entity1 = $this->createNode([
      'title' => 'Test Article',
      'type' => 'article',
    ]);
    $this->assertAccessAllowed($entity1);

    // Add our field:
    $entity2 = $this->createNode([
      'title' => 'Test Article2',
      'type' => 'article',
    ]);
    $this->setEntityAccessByRoleFieldOnEntity($entity2, $this->fieldName, ['anonymous'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity2);
  }

  /**
   * Check access on a node which allows test_role users, as a test_role user.
   */
  public function testAccessOnAllowedNodeAsNewRole() {
    $this->drupalLogin($this->testRoleUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['test_role'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity, 'view', $this->testRoleUser);
    // Now test with get request:
    $session = $this->assertSession();
    $this->drupalGet('/node/' . $entity->id());
    $session->statusCodeEquals(200);
  }

  /**
   * Check access on a node which forbids test_role users, as a test_role user.
   */
  public function testAccessOnForbiddenNodeAsNewRole() {
    $this->drupalLogin($this->testRoleUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['test_role'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity, 'view', $this->testRoleUser);
  }

  /**
   * Check access on an allowed node's operation edit.
   */
  public function testAccessOnAllowedNodeWithOperationEdit() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_EDIT], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity, Constants::OPERATION_EDIT);
  }

  /**
   * Check access on an forbidden node's operation edit.
   */
  public function testAccessOnForbiddenNodeWithOperationEdit() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_EDIT], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity, Constants::OPERATION_EDIT);
    // Now test with get request:
    $session = $this->assertSession();
    $this->drupalGet('/node/' . $entity->id() . '/edit');
    $session->statusCodeEquals(403);
  }

  /**
   * Check access on an allowed node's operation delete.
   */
  public function testAccessOnAllowedNodeWithOperationDelete() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_DELETE], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity, Constants::OPERATION_DELETE);
  }

  /**
   * Check access on an forbidden node's operation delete.
   */
  public function testAccessOnForbiddenNodeWithOperationDelete() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_DELETE], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity, Constants::OPERATION_DELETE);
    // Now test with get request:
    $session = $this->assertSession();
    $this->drupalGet('/node/' . $entity->id() . '/delete');
    $session->statusCodeEquals(403);
  }

  /**
   * Check access on an allowed node's operation via unpublished.
   */
  public function testAccessOnAllowedNodeWithOperationViewUnpublished() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW_UNPUBLISHED], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode([
      'title' => 'Test Article',
      'type' => 'article',
      'status' => '0',
    ]);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity, Constants::OPERATION_VIEW);
  }

  /**
   * Check view access on an allowed node which is unpublished.
   */
  public function testAccessOnAllowedUnpublishedNodeWithOperationView() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode([
      'title' => 'Test Article',
      'type' => 'article',
      'status' => '0',
    ]);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity, Constants::OPERATION_VIEW);
    // Now test with get request:
    $session = $this->assertSession();
    $this->drupalGet('/node/' . $entity->id());
    $session->statusCodeEquals(403);
  }

  /**
   * Check access on an forbidden node's operation vie unpublished.
   */
  public function testAccessOnForbiddenNodeWithOperationViewUnpublished() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW_UNPUBLISHED], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode([
      'title' => 'Test Article',
      'type' => 'article',
      'status' => '0',
    ]);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, ['authenticated'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity, Constants::OPERATION_VIEW);
    // Now test with get request:
    $session = $this->assertSession();
    $this->drupalGet('/node/' . $entity->id());
    $session->statusCodeEquals(403);
  }

  /**
   * Check access on a node which forbids test_role users, as a test_role user.
   */
  public function testAccessOnAllowedNodeWithFallbackAllowed() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_ALLOWED, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, [], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity, Constants::OPERATION_VIEW);
    // Now test with get request:
    $session = $this->assertSession();
    $this->drupalGet('/node/' . $entity->id());
    $session->statusCodeEquals(200);
  }

  /**
   * Check access on a node which forbids test_role users, as a test_role user.
   */
  public function testAccessOnForbiddenNodeWithFallbackAllowed() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_ALLOWED, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, [], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessAllowed($entity, Constants::OPERATION_VIEW);
    // Now test with get request:
    $session = $this->assertSession();
    $this->drupalGet('/node/' . $entity->id());
    $session->statusCodeEquals(200);
  }

  /**
   * Check access on a node which forbids test_role users, as a test_role user.
   */
  public function testAccessOnAllowedNodeWithFallbackForbidden() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_FORBIDDEN, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, [], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity, Constants::OPERATION_VIEW);
  }

  /**
   * Check access on a node which forbids test_role users, as a test_role user.
   */
  public function testAccessOnForbiddenNodeWithFallbackForbidden() {
    $this->drupalLogin($this->authenticatedUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_FORBIDDEN, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $entity = $this->createNode(['title' => 'Test Article', 'type' => 'article']);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($entity, $this->fieldName, [], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->assertAccessForbidden($entity, Constants::OPERATION_VIEW);
  }

  /**
   * Test to see if created roles are shown on creation of an article node.
   *
   * @todo This test is not working, fix it at some Point (remove todo in name).
   */
  public function todoTestNodeRoleFieldsExist() {
    $this->drupalLogin($this->adminUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    \Drupal::service('entity_field.manager')->clearCachedFieldDefinitions();
    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $display_repository */
    $display_repository = \Drupal::service('entity_display.repository');
    $display_repository->getFormDisplay('node', 'article')
      ->setComponent($this->fieldName, [
        'type' => 'default_entity_access_by_role_field_widget',
      ])
      ->save();
    $session = $this->assertSession();
    $this->drupalGet('/node/add/article');
    // $this->assertEquals('', $this->getSession()->getPage()->getHTML());
    $session->elementExists('css', "#edit-field-access-details-role-id-anonymous");
    $session->elementExists('css', "#edit-field-access-details-role-id-authenticated");
    $session->elementExists('css', "#edit-field-access-details-role-id-administrator");
    $session->elementExists('css', "#edit-field-access-details-role-id-role");
  }

  /**
   * Tests to see if the access fields exists.
   *
   * @todo This test is not working, fix it at some Point (remove todo in name).
   */
  public function todoTestNodeAccessFieldsExist() {
    $this->drupalLogin($this->adminUser);
    $entityNodeArticleBundle = $this->drupalCreateContentType(['type' => 'article']);
    $this->createAccessFieldTypeOnEntityBundle($entityNodeArticleBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    \Drupal::service('entity_field.manager')->clearCachedFieldDefinitions();
    $session = $this->assertSession();
    $this->drupalGet('/node/add/article');
    $session->elementExists('css', "#edit-field-access-details-access-allowed");
    $session->elementExists('css', "#edit-field-access-details-access-forbidden");
  }

}
